//
// automatically generated by spin2cpp v1.04 on Wed May  8 08:09:37 2013
// spin2cpp --ccode --files TestNunchuck.spin 
//

/*
┌──────────────────────────────────────────┐
│ Wii Nunchuck Driver Object v1.0          │
│ Author: Pat Daderko (DogP)               │               
│ Copyright (c) 2009                       │               
│ See end of file for terms of use.        │                
└──────────────────────────────────────────┘

Based on a Wii nunchuck example project from John Abshier, which was based on code originally by João Geada

This code uses the Floating Point library for computing pitch and roll.  If this data isn't needed, you can
save 2 cogs by removing this functionality.  You can probably also combine only the needed floating point
functionality into a single cog.

Note there is no yaw, as that can't be determined from the accelerometers.  This also limits the pitch
to 180 degrees and will cause incorrect roll readings when the pitch is between 180 deg and 360 deg.

Also note that the Nunchuck can't be constantly read, or bad data will be returned.  A wait can be added
at the end of the read function if you'd like to ensure it doesn't read too often, but that's probably
better left to the end user to prevent processing time from being unnecessarily wasted.

The CAL_X, CAL_Y, and CAL_Z values are calibration values for zeroing the acceleration and may need to be
adjusted, depending on your specific hardware.  This is the 0 value for no acceleration in its axis, not
the value being held still (when held still, zeroing two values should give a third value around 200 from
the force of gravity).  Find your center position, zero the two axes perpendicular to the force of gravity,
then rotate the third axis perpendicular to gravity and zero that axis.

The CAL_JOY_X and CAL_JOY_Y are the center values of the joystick.  128 seems like the nominal value, but
varies depending on the nunchuck.     

Diagram below is showing the pinout looking into the connector (which plugs into the Wii Remote)
 _______ 
| 1 2 3 |
|       |
| 6 5 4 |
|_-----_|

1 - SDA 
2 - 
3 - VCC
4 - SCL 
5 - 
6 - GND

This is an I2C peripheral, and requires a pullup resistor on the SDA line
If using a prop board with an I2C EEPROM, this can be connected directly to pin 28 (SCL) and pin 29 (SDA)
*/

//#include <propeller.h>
#include "Nunchuck.h"
#include "simpletools.h"

#ifdef __GNUC__
#define INLINE__ static inline
#define PostEffect__(X, Y) __extension__({ int32_t tmp__ = (X); (X) = (Y); tmp__; })
#else
#define INLINE__ static
static int32_t tmp__;
#define PostEffect__(X, Y) (tmp__ = (X), (X) = (Y), tmp__)
#define waitcnt(n) _waitcnt(n)
#define locknew() _locknew()
#define lockret(i) _lockret(i)
#define lockset(i) _lockset(i)
#define lockclr(i) _lockclr(i)
#define coginit(id, code, par) _coginit((unsigned)(par)>>2, (unsigned)(code)>>2, id)
#define cognew(code, par) coginit(0x8, (code), (par))
#define cogstop(i) _cogstop(i)
#endif

INLINE__ int32_t Shr__(uint32_t a, uint32_t b) { return (a>>b); }
static uint32_t Sqrt__(uint32_t a) {
    uint32_t res = 0;
    uint32_t bit = 1<<30;
    while (bit > a) bit = bit>>2;
    while (bit != 0) {
        if (a >= res+bit) {
            a = a - (res+bit);
            res = (res>>1) + bit;
        } else res = res >> 1;
        bit = bit >> 2;
    }
    return res;
}
static Nunchuck thisobj;
int32_t Nunchuck_Init(int32_t _Scl, int32_t _Sda)
{
  thisobj.I2cscl = _Scl;
  thisobj.I2csda = _Sda;
  i2cObject_Init(thisobj.I2csda, thisobj.I2cscl, 0);
//  Float32Full_Start();
  thisobj._220us = ((CLKFREQ / 100000) * 22);
  return 0;
}

int32_t Nunchuck_Readnunchuck(void)
{
  int32_t	Data[6];
  i2cObject_Writelocation(Nunchuck_addr, 240, 85, 8, 8);
  waitcnt((thisobj._220us + CNT));
  i2cObject_Writelocation(Nunchuck_addr, 251, 0, 8, 8);
  waitcnt((thisobj._220us + CNT));
  i2cObject_I2cstart();
  i2cObject_I2cwrite(Nunchuck_addr, 8);
  i2cObject_I2cwrite(0, 8);
  i2cObject_I2cstop();
  waitcnt((thisobj._220us + CNT));
  i2cObject_I2cstart();
  i2cObject_I2cwrite((0xa4 | 0x1), 8);
  Data[0] = i2cObject_I2cread(0);
  Data[1] = i2cObject_I2cread(0);
  Data[2] = i2cObject_I2cread(0);
  Data[3] = i2cObject_I2cread(0);
  Data[4] = i2cObject_I2cread(0);
  Data[5] = i2cObject_I2cread(1);
  i2cObject_I2cstop();
  thisobj.Joy_x = (Data[0] - Cal_joy_x);
  thisobj.Joy_y = (Data[1] - Cal_joy_y);
  thisobj.Accel_x = (((Data[2] << 2) | ((Shr__(Data[5], 2)) & 0x3)) - Cal_x);
  thisobj.Accel_y = (((Data[3] << 2) | ((Shr__(Data[5], 4)) & 0x3)) - Cal_y);
  thisobj.Accel_z = (((Data[4] << 2) | ((Shr__(Data[5], 6)) & 0x3)) - Cal_z);
  thisobj.Button_z = ((Data[5] & 0x1) ^ 0x1);
  thisobj.Button_c = (((Shr__(Data[5], 1)) & 0x1) ^ 0x1);
  return 0;
}

int32_t Nunchuck_Joyx(void)
{
  return thisobj.Joy_x;
}

int32_t Nunchuck_Joyy(void)
{
  return thisobj.Joy_y;
}

int32_t Nunchuck_Accelx(void)
{
  return thisobj.Accel_x;
}

int32_t Nunchuck_Accely(void)
{
  return thisobj.Accel_y;
}

int32_t Nunchuck_Accelz(void)
{
  return thisobj.Accel_z;
}

float Nunchuck_Radius(void)
{
  return sqrt(pow(thisobj.Accel_x,2) + pow(thisobj.Accel_y,2) + pow(thisobj.Accel_z,2));

//  As translated by spin2cpp:
//  return (sqrt((((thisobj.Accel_x * thisobj.Accel_x) + (thisobj.Accel_y * thisobj.Accel_y)) + (thisobj.Accel_z * thisobj.Accel_z))));
}

float Nunchuck_Pitch(void)
{
  float	radius;

  radius = Nunchuck_Radius();
  if (radius > 0) {
    return (((acos(thisobj.Accel_y / radius)) * 180.00 / PI) - 90);
  } else {
    return 0;
  }

//  As translated by spin2cpp:
//  float	Rad;
//
//  Rad = Nunchuck_Radius();
//  if (Rad > 0) {
//    return (Float32Full_Degrees(Float32Full_Acos(Float32Full_Fdiv(Float32Full_Ffloat(thisobj.Accel_y), Float32Full_Ffloat(Nunchuck_Radius()) ))) - 90);
//  } else {
//    return 0;
//  }
}
float Nunchuck_Roll(void)
{

  return (atan2(thisobj.Accel_x,thisobj.Accel_z) * 180.00 / PI);

//  As translated by spin2cpp:
//
//  return Float32Full_Degrees(Float32Full_Atan2(Float32Full_Ffloat(thisobj.Accel_x), Float32Full_Ffloat(thisobj.Accel_z)));
}

int32_t Nunchuck_Buttonc(void)
{
  return thisobj.Button_c;
}

int32_t Nunchuck_Buttonz(void)
{
  return thisobj.Button_z;
}

/*
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │                                                            
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
*/
